@@ -23,7 +23,7 @@ URL = 'https://erdogant.github.io/datasets/graphviz-2.38.zip'
 
 
 # %% Plot tree
-def plot(model, featnames=None, num_trees=0, plottype='horizontal', figsize=(25,25), verbose=3):
+def plot(model, featnames=None, num_trees=None, plottype='horizontal', figsize=(25,25), verbose=3):
     """Make tree plot for the input model.
 
     Parameters
@@ -32,8 +32,8 @@ def plot(model, featnames=None, num_trees=0, plottype='horizontal', figsize=(25,
         xgboost or randomforest model.
     featnames : list, optional
         list of feature names. The default is None.
-    num_trees : int, default 0
-        Specify the ordinal number of target tree
+    num_trees : int, default None
+        The best performing tree is choosen. Specify any other ordinal number for another target tree
     plottype : str, (default : 'horizontal')
         Works only in case of xgb model.
         * 'horizontal'
@@ -57,15 +57,54 @@ def plot(model, featnames=None, num_trees=0, plottype='horizontal', figsize=(25,
     elif ('tree' in modelname) or ('forest' in modelname) or ('gradientboosting' in modelname):
         if verbose>=4: print('tree plotting pipeline.')
         ax = randomforest(model, featnames=featnames, num_trees=num_trees, figsize=figsize, verbose=verbose)
+    elif ('lgb' in modelname):
+        ax = lgbm(model, featnames=featnames, num_trees=num_trees, figsize=figsize, verbose=verbose)
     else:
-        print('[treeplot] Model %s not recognized.' %(modelname))
+        print('[treeplot] >Model not recognized: %s' %(modelname))
         ax = None
 
     return ax
 
 
 # %% Plot tree
-def xgboost(model, featnames=None, num_trees=0, plottype='horizontal', figsize=(25,25), verbose=3):
+def lgbm(model, featnames=None, num_trees=None, figsize=(25,25), verbose=3):
+    try:
+        from lightgbm import plot_tree, plot_importance
+    except:
+        if verbose>=1: raise ImportError('lightgbm must be installed. Try to: <pip install lightgbm>')
+        return None
+
+    # Check model
+    _check_model(model, 'lgb')
+    # Set env
+    _set_graphviz_path()
+
+    if (num_trees is None) and hasattr(model, 'best_iteration_'):
+        num_trees = model.best_iteration_
+        if verbose>=3: print('[treeplot] >Best detected tree: %.0d' %(num_trees))
+    elif num_trees is None:
+        num_trees = 0
+    
+    ax1 = None
+    try:
+        fig, ax1 = plt.subplots(1, 1, figsize=figsize)
+        plot_tree(model, tree_index=num_trees, dpi=200, ax=ax1)
+    except:
+        if _get_platform() != "windows":
+            print('[treeplot] >Install graphviz first: <sudo apt install python-pydot python-pydot-ng graphviz>')
+
+    # Plot importance
+    ax2 = None
+    try:
+        fig, ax2 = plt.subplots(1, 1, figsize=figsize)
+        plot_importance(model, max_num_features=50, ax=ax2)
+    except:
+        print('[treeplot] >Error: importance can not be plotted. Booster.get_score() results in empty. This maybe caused by having all trees as decision dumps.')
+
+    return(ax1, ax2)
+
+# %% Plot tree
+def xgboost(model, featnames=None, num_trees=None, plottype='horizontal', figsize=(25,25), verbose=3):
     """Plot tree based on a xgboost.
 
     Parameters
@@ -74,8 +113,8 @@ def xgboost(model, featnames=None, num_trees=0, plottype='horizontal', figsize=(
         xgboost model.
     featnames : list, optional
         list of feature names. The default is None.
-    num_trees : int, default 0
-        Specify the ordinal number of target tree
+    num_trees : int, default None
+        The best performing tree is choosen. Specify any other ordinal number for another target tree
     plottype : str, optional
         Make 'horizontal' or 'vertical' plot. The default is 'horizontal'.
     figsize: tuple, default (25,25)
@@ -93,7 +132,7 @@ def xgboost(model, featnames=None, num_trees=0, plottype='horizontal', figsize=(
     try:
         from xgboost import plot_tree, plot_importance
     except:
-        raise ImportError('xgboost must be installed. Try to: <pip install xgboost>')
+        if verbose>=1: raise ImportError('xgboost must be installed. Try to: <pip install xgboost>')
 
     _check_model(model, 'xgb')
     # Set env
@@ -101,22 +140,33 @@ def xgboost(model, featnames=None, num_trees=0, plottype='horizontal', figsize=(
 
     if plottype=='horizontal': plottype='UD'
     if plottype=='vertical': plottype='LR'
+    if (num_trees is None) and hasattr(model, 'best_iteration'):
+        num_trees = model.best_iteration
+        if verbose>=3: print('[treeplot] >Best detected tree: %.0d' %(num_trees))
+    elif num_trees is None:
+        num_trees = 0
 
+    ax1 = None
     try:
-        fig, ax = plt.subplots(1, 1, figsize=figsize)
-        plot_tree(model, num_trees=num_trees, rankdir=plottype, ax=ax)
+        fig, ax1 = plt.subplots(1, 1, figsize=figsize)
+        plot_tree(model, num_trees=num_trees, rankdir=plottype, ax=ax1)
     except:
         if _get_platform() != "windows":
-            print('[TREEPLOT] Install graphviz first: <sudo apt install python-pydot python-pydot-ng graphviz>')
+            print('[treeplot] >Install graphviz first: <sudo apt install python-pydot python-pydot-ng graphviz>')
 
     # Plot importance
-    plot_importance(model)
+    ax2 = None
+    try:
+        fig, ax2 = plt.subplots(1, 1, figsize=figsize)
+        plot_importance(model, max_num_features=50, ax=ax2)
+    except:
+        print('[treeplot] >Error: importance can not be plotted. Booster.get_score() results in empty. This maybe caused by having all trees as decision dumps.')
 
-    return(ax)
+    return(ax1, ax2)
 
 
 # %% Plot tree
-def randomforest(model, featnames=None, num_trees=0, filepath='tree', export='png', resolution=100, figsize=(25,25), verbose=3):
+def randomforest(model, featnames=None, num_trees=None, filepath='tree', export='png', resolution=100, figsize=(25,25), verbose=3):
     """Plot tree based on a randomforest.
 
     Parameters
@@ -149,11 +199,13 @@ def randomforest(model, featnames=None, num_trees=0, filepath='tree', export='pn
     ax=None
     dotfile = None
     pngfile = None
+    if num_trees is None: num_trees = 0
+
     # Check model
     _check_model(model, 'randomforest')
     # Set env
     _set_graphviz_path()
-    
+
     if export is not None:
         dotfile = filepath + '.dot'
         pngfile = filepath + '.png'
@@ -196,7 +248,7 @@ def randomforest(model, featnames=None, num_trees=0, filepath='tree', export='pn
             plt.show()
         except:
             if _get_platform() != "windows":
-                print('[TREEPLOT] Install graphviz first: <sudo apt install python-pydot python-pydot-ng graphviz>')
+                print('[treeplot] >Install graphviz first: <sudo apt install python-pydot python-pydot-ng graphviz>')
     else:
         graph = Source(dot_data)
         plt.show()
@@ -259,7 +311,7 @@ def _set_graphviz_path(verbose=3):
         getZip = os.path.abspath(os.path.join(curpath, gfile))
         # Unzip if path does not exists
         if not os.path.isdir(getPath):
-            if verbose>=3: print('[TREEPLOT] Extracting graphviz files..')
+            if verbose>=3: print('[treeplot] >Extracting graphviz files..')
             [pathname, _] = os.path.split(getZip)
             # Unzip
             zip_ref = zipfile.ZipFile(getZip, 'r')
@@ -278,7 +330,7 @@ def _set_graphviz_path(verbose=3):
 
     # Add to system
     if finPath not in os.environ["PATH"]:
-        if verbose>=3: print('[TREEPLOT] Set path in environment.')
+        if verbose>=3: print('[treeplot] >Set path in environment.')
         os.environ["PATH"] += os.pathsep + finPath
 
     return(finPath)
@@ -304,11 +356,13 @@ def _check_model(model, expected):
         if ('forest' in modelname) or ('tree' in modelname) or ('gradientboosting' in modelname):
             pass
         else:
-            print('WARNING: The input model seems not to be a tree-based model?')
+            print('[treeplot] >>Warning: The input model seems not to be a tree-based model?')
     if (expected=='xgb'):
         if ('xgb' not in modelname):
-            print('WARNING: The input model seems not to be a xgboost model?')
-
+            print('[treeplot] >Warning: The input model seems not to be a xgboost model?')
+    if (expected=='lgb'):
+        if ('lgb' not in modelname):
+            print('[treeplot] >Warning: The input model seems not to be a lightgbm model?')
 
 # %% Import example dataset from github.
 def _download_graphviz(url, verbose=3):
@@ -332,13 +386,13 @@ def _download_graphviz(url, verbose=3):
     gfile = wget.filename_from_url(url)
     PATH_TO_DATA = os.path.join(curpath, gfile)
     if not os.path.isdir(curpath):
-        if verbose>=3: print('[treeplot] Downloading graphviz..')
+        if verbose>=3: print('[treeplot] >Downloading graphviz..')
         os.makedirs(curpath, exist_ok=True)
 
     # Check file exists.
     if not os.path.isfile(PATH_TO_DATA):
         # Download data from URL
-        if verbose>=3: print('[treeplot] Downloading graphviz..')
+        if verbose>=3: print('[treeplot] >Downloading graphviz..')
         wget.download(url, curpath)
 
     return(gfile, curpath)
